#ifndef lint
static char sccsid[] = "@(#)tiffinfo.c	1.2 2/19/89";
#endif

/*
 * Copyright (c) 1988 by Sam Leffler.
 * All rights reserved.
 *
 * This file is provided for unrestricted use provided that this
 * legend is included on all tape media and as a part of the
 * software program in whole or part.  Users may copy, modify or
 * distribute this file at will.
 */

#include <stdio.h>
#include <ctype.h>
#include "tiffio.h"

#define	streq(a,b)	(strcmp(a,b)==0)
#define	howmany(x, y)	(((x)+((y)-1))/(y))

int	showcolormap = 0;		/* show colormap entries if present */
int	showresponsecurves = 0;		/* show response curve[s] if present */
int	showstrips = 0;			/* show strip offsets and sizes */
int	showdata = 0;			/* show data */

main(argc, argv)
	char *argv[];
{
	int dirnum = -9999, multiplefiles;
	char *cp;
	TIFF *tif;

	argc--, argv++;
	while (argc > 0 && argv[0][0] == '-') {
		for (cp = &argv[0][1]; *cp; cp++) switch (*cp) {
		case '1': case '2': case '3': case '4': case '5':
		case '6': case '7': case '8': case '9':
			dirnum = atoi(cp);
			goto next;
		case 'd':
			showdata++;
			break;
		case 'c':
			showcolormap++;
			showresponsecurves++;
			break;
		case 's':
			showstrips++;
			break;
		default:
			usage(-1);
			/*NOTREACHED*/
		}
	next:
		argc--, argv++;
	}
	if (argc <= 0)
		usage(-2);
	for (multiplefiles = argc > 1; argc > 0; argc--, argv++) {
		if (multiplefiles)
			printf("%s:\n", argv[0]);
		tif = TIFFOpen(argv[0], "r");
		if (tif != NULL) {
			TIFFInfo(tif, dirnum);
			TIFFClose(tif);
		}
	}
	exit(0);
}

TIFFInfo(tif, dirnum)
	register TIFF *tif;
{
	TIFFDirectory *td;
	u_char *scanbuf, *pp;
	int bitspersample, scanline, samplesperbyte;
	register int cc, bytesperline, stride;
	int w, h, i;

	do {
		if (dirnum != -9999) {
			dirnum--;
			if (dirnum > 0)
				continue;
			if (dirnum < 0)
				break;
		}
		TIFFPrintDirectory(tif, stdout,
		    showstrips, showcolormap, showresponsecurves);
		td = tif->tif_dir;
		bitspersample = td->td_bitspersample;
		if (td->td_planarconfig == PLANARCONFIG_CONTIG)
			bitspersample *= td->td_samplesperpixel;
		scanline = howmany(bitspersample * td->td_imagewidth, 8);
		scanbuf = (u_char *)malloc(scanline);
		if (scanbuf == NULL) {
			fprintf(stderr, "No space for scanline buffer\n");
			continue;
		}
		w = td->td_imagewidth;
		h = td->td_imagelength;
		samplesperbyte =
		    bitspersample == 1 ? 8 : bitspersample == 4 ? 2 : 1;
		w /= samplesperbyte;
		stride = (td->td_planarconfig == PLANARCONFIG_CONTIG &&
		    td->td_samplesperpixel > 1 ? td->td_samplesperpixel-1 : 0);
		while (h-- > 0 && TIFFReadScanline(tif, scanbuf) > 0) {
			if (!showdata)
				continue;
			pp = scanbuf;
			for (cc = 0; cc < scanline && cc < w; cc++) {
				printf(" %02x", *pp++);
				if (((cc+1) % 24) == 0)
					putchar('\n');
				pp += stride;
			}
			putchar('\n');
		}
		free((char *)scanbuf);
	} while (TIFFReadDirectory(tif));
}

usage(code)
	int code;
{
	fprintf(stderr, "Usage: tiffinfo [-dcs] [-#] TIFF-files\n");
	fprintf(stderr, "-d    show data\n");
	fprintf(stderr, "-c    show color/gray response curves and colormap\n");
	fprintf(stderr, "-s    show data strip offsets and byte counts\n");
	fprintf(stderr, "-#    show directory number #\n");
	exit(code);
}
